# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.onchain.networks.pools import trade_get_params
from .....types.onchain.networks.pools.trade_get_response import TradeGetResponse

__all__ = ["TradesResource", "AsyncTradesResource"]


class TradesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TradesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return TradesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TradesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return TradesResourceWithStreamingResponse(self)

    def get(
        self,
        pool_address: str,
        *,
        network: str,
        token: str | NotGiven = NOT_GIVEN,
        trade_volume_in_usd_greater_than: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TradeGetResponse:
        """
        This endpoint allows you to **query the last 300 trades in the past 24 hours
        based on the provided pool address**

        Args:
          token: return trades for token use this to invert the chart Available values: 'base',
              'quote' or token address Default value: 'base'

          trade_volume_in_usd_greater_than: filter trades by trade volume in USD greater than this value Default value: 0

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not pool_address:
            raise ValueError(f"Expected a non-empty value for `pool_address` but received {pool_address!r}")
        return self._get(
            f"/onchain/networks/{network}/pools/{pool_address}/trades",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "token": token,
                        "trade_volume_in_usd_greater_than": trade_volume_in_usd_greater_than,
                    },
                    trade_get_params.TradeGetParams,
                ),
            ),
            cast_to=TradeGetResponse,
        )


class AsyncTradesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTradesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTradesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTradesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncTradesResourceWithStreamingResponse(self)

    async def get(
        self,
        pool_address: str,
        *,
        network: str,
        token: str | NotGiven = NOT_GIVEN,
        trade_volume_in_usd_greater_than: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TradeGetResponse:
        """
        This endpoint allows you to **query the last 300 trades in the past 24 hours
        based on the provided pool address**

        Args:
          token: return trades for token use this to invert the chart Available values: 'base',
              'quote' or token address Default value: 'base'

          trade_volume_in_usd_greater_than: filter trades by trade volume in USD greater than this value Default value: 0

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not pool_address:
            raise ValueError(f"Expected a non-empty value for `pool_address` but received {pool_address!r}")
        return await self._get(
            f"/onchain/networks/{network}/pools/{pool_address}/trades",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "token": token,
                        "trade_volume_in_usd_greater_than": trade_volume_in_usd_greater_than,
                    },
                    trade_get_params.TradeGetParams,
                ),
            ),
            cast_to=TradeGetResponse,
        )


class TradesResourceWithRawResponse:
    def __init__(self, trades: TradesResource) -> None:
        self._trades = trades

        self.get = to_raw_response_wrapper(
            trades.get,
        )


class AsyncTradesResourceWithRawResponse:
    def __init__(self, trades: AsyncTradesResource) -> None:
        self._trades = trades

        self.get = async_to_raw_response_wrapper(
            trades.get,
        )


class TradesResourceWithStreamingResponse:
    def __init__(self, trades: TradesResource) -> None:
        self._trades = trades

        self.get = to_streamed_response_wrapper(
            trades.get,
        )


class AsyncTradesResourceWithStreamingResponse:
    def __init__(self, trades: AsyncTradesResource) -> None:
        self._trades = trades

        self.get = async_to_streamed_response_wrapper(
            trades.get,
        )



import operator
from typing import Any
from typing_extensions import override

from coingecko_sdk._utils import LazyProxy


class RecursiveLazyProxy(LazyProxy[Any]):
    @override
    def __load__(self) -> Any:
        return self

    def __call__(self, *_args: Any, **_kwds: Any) -> Any:
        raise RuntimeError("This should never be called!")


def test_recursive_proxy() -> None:
    proxy = RecursiveLazyProxy()
    assert repr(proxy) == "RecursiveLazyProxy"
    assert str(proxy) == "RecursiveLazyProxy"
    assert dir(proxy) == []
    assert type(proxy).__name__ == "RecursiveLazyProxy"
    assert type(operator.attrgetter("name.foo.bar.baz")(proxy)).__name__ == "RecursiveLazyProxy"


def test_isinstance_does_not_error() -> None:
    class AlwaysErrorProxy(LazyProxy[Any]):
        @override
        def __load__(self) -> Any:
            raise RuntimeError("Mocking missing dependency")

    proxy = AlwaysErrorProxy()
    assert not isinstance(proxy, dict)
    assert isinstance(proxy, LazyProxy)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TradeGetParams"]


class TradeGetParams(TypedDict, total=False):
    network: Required[str]

    token: str
    """
    return trades for token use this to invert the chart Available values: 'base',
    'quote' or token address Default value: 'base'
    """

    trade_volume_in_usd_greater_than: float
    """filter trades by trade volume in USD greater than this value Default value: 0"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["MarketChartGetRangeParams"]


class MarketChartGetRangeParams(TypedDict, total=False):
    id: Required[str]

    from_: Required[Annotated[float, PropertyInfo(alias="from")]]
    """starting date in UNIX timestamp"""

    to: Required[float]
    """ending date in UNIX timestamp"""

    vs_currency: Required[str]
    """
    target currency of market data \\**refers to
    [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).
    """

    interval: Literal["5m", "hourly", "daily"]
    """data interval, leave empty for auto granularity"""

    precision: Literal[
        "full", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18"
    ]
    """decimal place for currency price value"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ExchangeGetParams"]


class ExchangeGetParams(TypedDict, total=False):
    order: Literal[
        "name_asc",
        "name_desc",
        "open_interest_btc_asc",
        "open_interest_btc_desc",
        "trade_volume_24h_btc_asc",
        "trade_volume_24h_btc_desc",
    ]
    """use this to sort the order of responses, default: open_interest_btc_desc"""

    page: float
    """page through results, default: 1"""

    per_page: float
    """total results per page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["OhlcGetRangeParams"]


class OhlcGetRangeParams(TypedDict, total=False):
    from_: Required[Annotated[float, PropertyInfo(alias="from")]]
    """starting date in UNIX timestamp"""

    interval: Required[Literal["daily", "hourly"]]
    """data interval"""

    to: Required[float]
    """ending date in UNIX timestamp"""

    vs_currency: Required[str]
    """
    target currency of price data \\**refers to
    [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = [
    "ExchangeGetIDResponse",
    "Ticker",
    "TickerTicker",
    "TickerTickerConvertedLast",
    "TickerTickerConvertedVolume",
    "TickerTickerMarket",
]


class TickerTickerConvertedLast(BaseModel):
    btc: Optional[float] = None

    eth: Optional[float] = None

    usd: Optional[float] = None


class TickerTickerConvertedVolume(BaseModel):
    btc: Optional[float] = None

    eth: Optional[float] = None

    usd: Optional[float] = None


class TickerTickerMarket(BaseModel):
    has_trading_incentive: bool
    """exchange trading incentive"""

    identifier: str
    """exchange identifier"""

    name: str
    """exchange name"""

    logo: Optional[str] = None
    """exchange image url"""


class TickerTicker(BaseModel):
    base: Optional[str] = None
    """coin ticker base currency"""

    bid_ask_spread_percentage: Optional[float] = None
    """coin ticker bid ask spread percentage"""

    coin_id: Optional[str] = None
    """coin ticker base currency coin ID"""

    converted_last: Optional[TickerTickerConvertedLast] = None
    """coin ticker converted last price"""

    converted_volume: Optional[TickerTickerConvertedVolume] = None
    """coin ticker converted volume"""

    cost_to_move_down_usd: Optional[float] = None
    """coin ticker cost to move down in usd"""

    cost_to_move_up_usd: Optional[float] = None
    """coin ticker cost to move up in usd"""

    is_anomaly: Optional[bool] = None
    """coin ticker anomaly"""

    is_stale: Optional[bool] = None
    """coin ticker stale"""

    last: Optional[float] = None
    """coin ticker last price"""

    last_fetch_at: Optional[str] = None
    """coin ticker last fetch timestamp"""

    last_traded_at: Optional[str] = None
    """coin ticker last traded timestamp"""

    market: Optional[TickerTickerMarket] = None
    """coin ticker exchange"""

    target: Optional[str] = None
    """coin ticker target currency"""

    target_coin_id: Optional[str] = None
    """coin ticker target currency coin ID"""

    timestamp: Optional[str] = None
    """coin ticker timestamp"""

    token_info_url: Optional[str] = None
    """coin ticker token info url"""

    trade_url: Optional[str] = None
    """coin ticker trade url"""

    trust_score: Optional[str] = None
    """coin ticker trust score"""

    volume: Optional[float] = None
    """coin ticker volume"""


class Ticker(BaseModel):
    name: Optional[str] = None
    """coin name"""

    tickers: Optional[List[TickerTicker]] = None
    """list of tickers"""


class ExchangeGetIDResponse(BaseModel):
    alert_notice: Optional[str] = None
    """alert notice for exchange"""

    centralized: Optional[bool] = None
    """exchange type (true for centralized, false for decentralized)"""

    coins: Optional[float] = None
    """number of coins listed on the exchange"""

    country: Optional[str] = None
    """exchange incorporated country"""

    description: Optional[str] = None
    """exchange description"""

    facebook_url: Optional[str] = None
    """exchange facebook url"""

    has_trading_incentive: Optional[bool] = None
    """exchange trading incentive"""

    image: Optional[str] = None
    """exchange image url"""

    name: Optional[str] = None
    """exchange name"""

    other_url_1: Optional[str] = None

    other_url_2: Optional[str] = None

    pairs: Optional[float] = None
    """number of trading pairs on the exchange"""

    public_notice: Optional[str] = None
    """public notice for exchange"""

    reddit_url: Optional[str] = None
    """exchange reddit url"""

    slack_url: Optional[str] = None
    """exchange slack url"""

    telegram_url: Optional[str] = None
    """exchange telegram url"""

    tickers: Optional[List[Ticker]] = None

    trade_volume_24h_btc: Optional[float] = None

    trust_score: Optional[float] = None
    """exchange trust score"""

    trust_score_rank: Optional[float] = None
    """exchange trust score rank"""

    twitter_handle: Optional[str] = None
    """exchange twitter handle"""

    url: Optional[str] = None
    """exchange website url"""

    year_established: Optional[float] = None
    """exchange established year"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["TickerGetParams"]


class TickerGetParams(TypedDict, total=False):
    coin_ids: str
    """
    filter tickers by coin IDs, comma-separated if querying more than 1 coin
    \\**refers to [`/coins/list`](/reference/coins-list).
    """

    depth: bool
    """
    include 2% orderbook depth (Example: cost_to_move_up_usd &
    cost_to_move_down_usd),default: false
    """

    dex_pair_format: Literal["contract_address", "symbol"]
    """
    set to `symbol` to display DEX pair base and target as symbols, default:
    `contract_address`
    """

    include_exchange_logo: bool
    """include exchange logo, default: false"""

    order: Literal["trust_score_desc", "trust_score_asc", "volume_desc", "volume_asc", "base_target"]
    """use this to sort the order of responses, default: trust_score_desc"""

    page: float
    """page through results"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ...._compat import cached_property
from .megafilter import (
    MegafilterResource,
    AsyncMegafilterResource,
    MegafilterResourceWithRawResponse,
    AsyncMegafilterResourceWithRawResponse,
    MegafilterResourceWithStreamingResponse,
    AsyncMegafilterResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from .trending_search import (
    TrendingSearchResource,
    AsyncTrendingSearchResource,
    TrendingSearchResourceWithRawResponse,
    AsyncTrendingSearchResourceWithRawResponse,
    TrendingSearchResourceWithStreamingResponse,
    AsyncTrendingSearchResourceWithStreamingResponse,
)

__all__ = ["PoolsResource", "AsyncPoolsResource"]


class PoolsResource(SyncAPIResource):
    @cached_property
    def megafilter(self) -> MegafilterResource:
        return MegafilterResource(self._client)

    @cached_property
    def trending_search(self) -> TrendingSearchResource:
        return TrendingSearchResource(self._client)

    @cached_property
    def with_raw_response(self) -> PoolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return PoolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PoolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return PoolsResourceWithStreamingResponse(self)


class AsyncPoolsResource(AsyncAPIResource):
    @cached_property
    def megafilter(self) -> AsyncMegafilterResource:
        return AsyncMegafilterResource(self._client)

    @cached_property
    def trending_search(self) -> AsyncTrendingSearchResource:
        return AsyncTrendingSearchResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPoolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPoolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPoolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncPoolsResourceWithStreamingResponse(self)


class PoolsResourceWithRawResponse:
    def __init__(self, pools: PoolsResource) -> None:
        self._pools = pools

    @cached_property
    def megafilter(self) -> MegafilterResourceWithRawResponse:
        return MegafilterResourceWithRawResponse(self._pools.megafilter)

    @cached_property
    def trending_search(self) -> TrendingSearchResourceWithRawResponse:
        return TrendingSearchResourceWithRawResponse(self._pools.trending_search)


class AsyncPoolsResourceWithRawResponse:
    def __init__(self, pools: AsyncPoolsResource) -> None:
        self._pools = pools

    @cached_property
    def megafilter(self) -> AsyncMegafilterResourceWithRawResponse:
        return AsyncMegafilterResourceWithRawResponse(self._pools.megafilter)

    @cached_property
    def trending_search(self) -> AsyncTrendingSearchResourceWithRawResponse:
        return AsyncTrendingSearchResourceWithRawResponse(self._pools.trending_search)


class PoolsResourceWithStreamingResponse:
    def __init__(self, pools: PoolsResource) -> None:
        self._pools = pools

    @cached_property
    def megafilter(self) -> MegafilterResourceWithStreamingResponse:
        return MegafilterResourceWithStreamingResponse(self._pools.megafilter)

    @cached_property
    def trending_search(self) -> TrendingSearchResourceWithStreamingResponse:
        return TrendingSearchResourceWithStreamingResponse(self._pools.trending_search)


class AsyncPoolsResourceWithStreamingResponse:
    def __init__(self, pools: AsyncPoolsResource) -> None:
        self._pools = pools

    @cached_property
    def megafilter(self) -> AsyncMegafilterResourceWithStreamingResponse:
        return AsyncMegafilterResourceWithStreamingResponse(self._pools.megafilter)

    @cached_property
    def trending_search(self) -> AsyncTrendingSearchResourceWithStreamingResponse:
        return AsyncTrendingSearchResourceWithStreamingResponse(self._pools.trending_search)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .pools import (
    PoolsResource,
    AsyncPoolsResource,
    PoolsResourceWithRawResponse,
    AsyncPoolsResourceWithRawResponse,
    PoolsResourceWithStreamingResponse,
    AsyncPoolsResourceWithStreamingResponse,
)
from .search import (
    SearchResource,
    AsyncSearchResource,
    SearchResourceWithRawResponse,
    AsyncSearchResourceWithRawResponse,
    SearchResourceWithStreamingResponse,
    AsyncSearchResourceWithStreamingResponse,
)
from .simple import (
    SimpleResource,
    AsyncSimpleResource,
    SimpleResourceWithRawResponse,
    AsyncSimpleResourceWithRawResponse,
    SimpleResourceWithStreamingResponse,
    AsyncSimpleResourceWithStreamingResponse,
)
from .tokens import (
    TokensResource,
    AsyncTokensResource,
    TokensResourceWithRawResponse,
    AsyncTokensResourceWithRawResponse,
    TokensResourceWithStreamingResponse,
    AsyncTokensResourceWithStreamingResponse,
)
from .onchain import (
    OnchainResource,
    AsyncOnchainResource,
    OnchainResourceWithRawResponse,
    AsyncOnchainResourceWithRawResponse,
    OnchainResourceWithStreamingResponse,
    AsyncOnchainResourceWithStreamingResponse,
)
from .networks import (
    NetworksResource,
    AsyncNetworksResource,
    NetworksResourceWithRawResponse,
    AsyncNetworksResourceWithRawResponse,
    NetworksResourceWithStreamingResponse,
    AsyncNetworksResourceWithStreamingResponse,
)
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)

__all__ = [
    "CategoriesResource",
    "AsyncCategoriesResource",
    "CategoriesResourceWithRawResponse",
    "AsyncCategoriesResourceWithRawResponse",
    "CategoriesResourceWithStreamingResponse",
    "AsyncCategoriesResourceWithStreamingResponse",
    "NetworksResource",
    "AsyncNetworksResource",
    "NetworksResourceWithRawResponse",
    "AsyncNetworksResourceWithRawResponse",
    "NetworksResourceWithStreamingResponse",
    "AsyncNetworksResourceWithStreamingResponse",
    "PoolsResource",
    "AsyncPoolsResource",
    "PoolsResourceWithRawResponse",
    "AsyncPoolsResourceWithRawResponse",
    "PoolsResourceWithStreamingResponse",
    "AsyncPoolsResourceWithStreamingResponse",
    "SearchResource",
    "AsyncSearchResource",
    "SearchResourceWithRawResponse",
    "AsyncSearchResourceWithRawResponse",
    "SearchResourceWithStreamingResponse",
    "AsyncSearchResourceWithStreamingResponse",
    "SimpleResource",
    "AsyncSimpleResource",
    "SimpleResourceWithRawResponse",
    "AsyncSimpleResourceWithRawResponse",
    "SimpleResourceWithStreamingResponse",
    "AsyncSimpleResourceWithStreamingResponse",
    "TokensResource",
    "AsyncTokensResource",
    "TokensResourceWithRawResponse",
    "AsyncTokensResourceWithRawResponse",
    "TokensResourceWithStreamingResponse",
    "AsyncTokensResourceWithStreamingResponse",
    "OnchainResource",
    "AsyncOnchainResource",
    "OnchainResourceWithRawResponse",
    "AsyncOnchainResourceWithRawResponse",
    "OnchainResourceWithStreamingResponse",
    "AsyncOnchainResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .market_chart import (
    MarketChartResource,
    AsyncMarketChartResource,
    MarketChartResourceWithRawResponse,
    AsyncMarketChartResourceWithRawResponse,
    MarketChartResourceWithStreamingResponse,
    AsyncMarketChartResourceWithStreamingResponse,
)
from ...._base_client import make_request_options
from ....types.coins.contract_get_response import ContractGetResponse

__all__ = ["ContractResource", "AsyncContractResource"]


class ContractResource(SyncAPIResource):
    @cached_property
    def market_chart(self) -> MarketChartResource:
        return MarketChartResource(self._client)

    @cached_property
    def with_raw_response(self) -> ContractResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return ContractResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContractResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return ContractResourceWithStreamingResponse(self)

    def get(
        self,
        contract_address: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractGetResponse:
        """
        This endpoint allows you to **query all the metadata (image, websites, socials,
        description, contract address, etc.) and market data (price, ATH, exchange
        tickers, etc.) of a coin from the CoinGecko coin page based on an asset platform
        and a particular token contract address**

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not contract_address:
            raise ValueError(f"Expected a non-empty value for `contract_address` but received {contract_address!r}")
        return self._get(
            f"/coins/{id}/contract/{contract_address}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractGetResponse,
        )


class AsyncContractResource(AsyncAPIResource):
    @cached_property
    def market_chart(self) -> AsyncMarketChartResource:
        return AsyncMarketChartResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncContractResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncContractResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContractResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncContractResourceWithStreamingResponse(self)

    async def get(
        self,
        contract_address: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ContractGetResponse:
        """
        This endpoint allows you to **query all the metadata (image, websites, socials,
        description, contract address, etc.) and market data (price, ATH, exchange
        tickers, etc.) of a coin from the CoinGecko coin page based on an asset platform
        and a particular token contract address**

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not contract_address:
            raise ValueError(f"Expected a non-empty value for `contract_address` but received {contract_address!r}")
        return await self._get(
            f"/coins/{id}/contract/{contract_address}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContractGetResponse,
        )


class ContractResourceWithRawResponse:
    def __init__(self, contract: ContractResource) -> None:
        self._contract = contract

        self.get = to_raw_response_wrapper(
            contract.get,
        )

    @cached_property
    def market_chart(self) -> MarketChartResourceWithRawResponse:
        return MarketChartResourceWithRawResponse(self._contract.market_chart)


class AsyncContractResourceWithRawResponse:
    def __init__(self, contract: AsyncContractResource) -> None:
        self._contract = contract

        self.get = async_to_raw_response_wrapper(
            contract.get,
        )

    @cached_property
    def market_chart(self) -> AsyncMarketChartResourceWithRawResponse:
        return AsyncMarketChartResourceWithRawResponse(self._contract.market_chart)


class ContractResourceWithStreamingResponse:
    def __init__(self, contract: ContractResource) -> None:
        self._contract = contract

        self.get = to_streamed_response_wrapper(
            contract.get,
        )

    @cached_property
    def market_chart(self) -> MarketChartResourceWithStreamingResponse:
        return MarketChartResourceWithStreamingResponse(self._contract.market_chart)


class AsyncContractResourceWithStreamingResponse:
    def __init__(self, contract: AsyncContractResource) -> None:
        self._contract = contract

        self.get = async_to_streamed_response_wrapper(
            contract.get,
        )

    @cached_property
    def market_chart(self) -> AsyncMarketChartResourceWithStreamingResponse:
        return AsyncMarketChartResourceWithStreamingResponse(self._contract.market_chart)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ....._compat import cached_property
from .token_price import (
    TokenPriceResource,
    AsyncTokenPriceResource,
    TokenPriceResourceWithRawResponse,
    AsyncTokenPriceResourceWithRawResponse,
    TokenPriceResourceWithStreamingResponse,
    AsyncTokenPriceResourceWithStreamingResponse,
)
from ....._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["NetworksResource", "AsyncNetworksResource"]


class NetworksResource(SyncAPIResource):
    @cached_property
    def token_price(self) -> TokenPriceResource:
        return TokenPriceResource(self._client)

    @cached_property
    def with_raw_response(self) -> NetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return NetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return NetworksResourceWithStreamingResponse(self)


class AsyncNetworksResource(AsyncAPIResource):
    @cached_property
    def token_price(self) -> AsyncTokenPriceResource:
        return AsyncTokenPriceResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncNetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncNetworksResourceWithStreamingResponse(self)


class NetworksResourceWithRawResponse:
    def __init__(self, networks: NetworksResource) -> None:
        self._networks = networks

    @cached_property
    def token_price(self) -> TokenPriceResourceWithRawResponse:
        return TokenPriceResourceWithRawResponse(self._networks.token_price)


class AsyncNetworksResourceWithRawResponse:
    def __init__(self, networks: AsyncNetworksResource) -> None:
        self._networks = networks

    @cached_property
    def token_price(self) -> AsyncTokenPriceResourceWithRawResponse:
        return AsyncTokenPriceResourceWithRawResponse(self._networks.token_price)


class NetworksResourceWithStreamingResponse:
    def __init__(self, networks: NetworksResource) -> None:
        self._networks = networks

    @cached_property
    def token_price(self) -> TokenPriceResourceWithStreamingResponse:
        return TokenPriceResourceWithStreamingResponse(self._networks.token_price)


class AsyncNetworksResourceWithStreamingResponse:
    def __init__(self, networks: AsyncNetworksResource) -> None:
        self._networks = networks

    @cached_property
    def token_price(self) -> AsyncTokenPriceResourceWithStreamingResponse:
        return AsyncTokenPriceResourceWithStreamingResponse(self._networks.token_price)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ExchangeGetListParams"]


class ExchangeGetListParams(TypedDict, total=False):
    status: Literal["active", "inactive"]
    """filter by status of exchanges, default: active"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .simple import (
    SimpleResource,
    AsyncSimpleResource,
    SimpleResourceWithRawResponse,
    AsyncSimpleResourceWithRawResponse,
    SimpleResourceWithStreamingResponse,
    AsyncSimpleResourceWithStreamingResponse,
)
from .networks import (
    NetworksResource,
    AsyncNetworksResource,
    NetworksResourceWithRawResponse,
    AsyncNetworksResourceWithRawResponse,
    NetworksResourceWithStreamingResponse,
    AsyncNetworksResourceWithStreamingResponse,
)

__all__ = [
    "NetworksResource",
    "AsyncNetworksResource",
    "NetworksResourceWithRawResponse",
    "AsyncNetworksResourceWithRawResponse",
    "NetworksResourceWithStreamingResponse",
    "AsyncNetworksResourceWithStreamingResponse",
    "SimpleResource",
    "AsyncSimpleResource",
    "SimpleResourceWithRawResponse",
    "AsyncSimpleResourceWithRawResponse",
    "SimpleResourceWithStreamingResponse",
    "AsyncSimpleResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel

__all__ = ["MarketChartGetResponse"]


class MarketChartGetResponse(BaseModel):
    floor_price_native: Optional[List[List[float]]] = None
    """NFT collection floor price in native currency"""

    floor_price_usd: Optional[List[List[float]]] = None
    """NFT collection floor price in usd"""

    h24_volume_native: Optional[List[List[float]]] = None
    """NFT collection volume in 24 hours in native currency"""

    h24_volume_usd: Optional[List[List[float]]] = None
    """NFT collection volume in 24 hours in usd"""

    market_cap_native: Optional[List[List[float]]] = None
    """NFT collection market cap in native currency"""

    market_cap_usd: Optional[List[List[float]]] = None
    """NFT collection market cap in usd"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from .._models import BaseModel

__all__ = ["AssetPlatformGetResponse", "AssetPlatformGetResponseItem", "AssetPlatformGetResponseItemImage"]


class AssetPlatformGetResponseItemImage(BaseModel):
    large: Optional[str] = None

    small: Optional[str] = None

    thumb: Optional[str] = None


class AssetPlatformGetResponseItem(BaseModel):
    id: Optional[str] = None
    """asset platform ID"""

    chain_identifier: Optional[float] = None
    """chainlist's chain ID"""

    image: Optional[AssetPlatformGetResponseItemImage] = None
    """image of the asset platform"""

    name: Optional[str] = None
    """chain name"""

    native_coin_id: Optional[str] = None
    """chain native coin ID"""

    shortname: Optional[str] = None
    """chain shortname"""


AssetPlatformGetResponse: TypeAlias = List[AssetPlatformGetResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.onchain.networks.pools import ohlcv_get_timeframe_params
from .....types.onchain.networks.pools.ohlcv_get_timeframe_response import OhlcvGetTimeframeResponse

__all__ = ["OhlcvResource", "AsyncOhlcvResource"]


class OhlcvResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OhlcvResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return OhlcvResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OhlcvResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return OhlcvResourceWithStreamingResponse(self)

    def get_timeframe(
        self,
        timeframe: Literal["day", "hour", "minute"],
        *,
        network: str,
        pool_address: str,
        token: str | NotGiven = NOT_GIVEN,
        aggregate: str | NotGiven = NOT_GIVEN,
        before_timestamp: int | NotGiven = NOT_GIVEN,
        currency: Literal["usd", "token"] | NotGiven = NOT_GIVEN,
        include_empty_intervals: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OhlcvGetTimeframeResponse:
        """
        This endpoint allows you to **get the OHLCV chart (Open, High, Low, Close,
        Volume) of a pool based on the provided pool address on a network**

        Args:
          token: return OHLCV for token use this to invert the chart Available values: 'base',
              'quote' or token address Default value: 'base'

          aggregate: time period to aggregate each OHLCV Available values (day): `1` Available values
              (hour): `1` , `4` , `12` Available values (minute): `1` , `5` , `15` Default
              value: 1

          before_timestamp: return OHLCV data before this timestamp (integer seconds since epoch)

          currency: return OHLCV in USD or quote token Default value: usd

          include_empty_intervals: include empty intervals with no trade data, default: false

          limit: number of OHLCV results to return, maximum 1000 Default value: 100

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not pool_address:
            raise ValueError(f"Expected a non-empty value for `pool_address` but received {pool_address!r}")
        if not timeframe:
            raise ValueError(f"Expected a non-empty value for `timeframe` but received {timeframe!r}")
        return self._get(
            f"/onchain/networks/{network}/pools/{pool_address}/ohlcv/{timeframe}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "token": token,
                        "aggregate": aggregate,
                        "before_timestamp": before_timestamp,
                        "currency": currency,
                        "include_empty_intervals": include_empty_intervals,
                        "limit": limit,
                    },
                    ohlcv_get_timeframe_params.OhlcvGetTimeframeParams,
                ),
            ),
            cast_to=OhlcvGetTimeframeResponse,
        )


class AsyncOhlcvResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOhlcvResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOhlcvResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOhlcvResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncOhlcvResourceWithStreamingResponse(self)

    async def get_timeframe(
        self,
        timeframe: Literal["day", "hour", "minute"],
        *,
        network: str,
        pool_address: str,
        token: str | NotGiven = NOT_GIVEN,
        aggregate: str | NotGiven = NOT_GIVEN,
        before_timestamp: int | NotGiven = NOT_GIVEN,
        currency: Literal["usd", "token"] | NotGiven = NOT_GIVEN,
        include_empty_intervals: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OhlcvGetTimeframeResponse:
        """
        This endpoint allows you to **get the OHLCV chart (Open, High, Low, Close,
        Volume) of a pool based on the provided pool address on a network**

        Args:
          token: return OHLCV for token use this to invert the chart Available values: 'base',
              'quote' or token address Default value: 'base'

          aggregate: time period to aggregate each OHLCV Available values (day): `1` Available values
              (hour): `1` , `4` , `12` Available values (minute): `1` , `5` , `15` Default
              value: 1

          before_timestamp: return OHLCV data before this timestamp (integer seconds since epoch)

          currency: return OHLCV in USD or quote token Default value: usd

          include_empty_intervals: include empty intervals with no trade data, default: false

          limit: number of OHLCV results to return, maximum 1000 Default value: 100

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not pool_address:
            raise ValueError(f"Expected a non-empty value for `pool_address` but received {pool_address!r}")
        if not timeframe:
            raise ValueError(f"Expected a non-empty value for `timeframe` but received {timeframe!r}")
        return await self._get(
            f"/onchain/networks/{network}/pools/{pool_address}/ohlcv/{timeframe}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "token": token,
                        "aggregate": aggregate,
                        "before_timestamp": before_timestamp,
                        "currency": currency,
                        "include_empty_intervals": include_empty_intervals,
                        "limit": limit,
                    },
                    ohlcv_get_timeframe_params.OhlcvGetTimeframeParams,
                ),
            ),
            cast_to=OhlcvGetTimeframeResponse,
        )


class OhlcvResourceWithRawResponse:
    def __init__(self, ohlcv: OhlcvResource) -> None:
        self._ohlcv = ohlcv

        self.get_timeframe = to_raw_response_wrapper(
            ohlcv.get_timeframe,
        )


class AsyncOhlcvResourceWithRawResponse:
    def __init__(self, ohlcv: AsyncOhlcvResource) -> None:
        self._ohlcv = ohlcv

        self.get_timeframe = async_to_raw_response_wrapper(
            ohlcv.get_timeframe,
        )


class OhlcvResourceWithStreamingResponse:
    def __init__(self, ohlcv: OhlcvResource) -> None:
        self._ohlcv = ohlcv

        self.get_timeframe = to_streamed_response_wrapper(
            ohlcv.get_timeframe,
        )


class AsyncOhlcvResourceWithStreamingResponse:
    def __init__(self, ohlcv: AsyncOhlcvResource) -> None:
        self._ohlcv = ohlcv

        self.get_timeframe = async_to_streamed_response_wrapper(
            ohlcv.get_timeframe,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.token_list_get_all_json_response import TokenListGetAllJsonResponse

__all__ = ["TokenListsResource", "AsyncTokenListsResource"]


class TokenListsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TokenListsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return TokenListsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TokenListsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return TokenListsResourceWithStreamingResponse(self)

    def get_all_json(
        self,
        asset_platform_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TokenListGetAllJsonResponse:
        """
        This endpoint allows you to **get full list of tokens of a blockchain network
        (asset platform) that is supported by
        [Ethereum token list standard](https://tokenlists.org/)**

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not asset_platform_id:
            raise ValueError(f"Expected a non-empty value for `asset_platform_id` but received {asset_platform_id!r}")
        return self._get(
            f"/token_lists/{asset_platform_id}/all.json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TokenListGetAllJsonResponse,
        )


class AsyncTokenListsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTokenListsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTokenListsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTokenListsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncTokenListsResourceWithStreamingResponse(self)

    async def get_all_json(
        self,
        asset_platform_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TokenListGetAllJsonResponse:
        """
        This endpoint allows you to **get full list of tokens of a blockchain network
        (asset platform) that is supported by
        [Ethereum token list standard](https://tokenlists.org/)**

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not asset_platform_id:
            raise ValueError(f"Expected a non-empty value for `asset_platform_id` but received {asset_platform_id!r}")
        return await self._get(
            f"/token_lists/{asset_platform_id}/all.json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TokenListGetAllJsonResponse,
        )


class TokenListsResourceWithRawResponse:
    def __init__(self, token_lists: TokenListsResource) -> None:
        self._token_lists = token_lists

        self.get_all_json = to_raw_response_wrapper(
            token_lists.get_all_json,
        )


class AsyncTokenListsResourceWithRawResponse:
    def __init__(self, token_lists: AsyncTokenListsResource) -> None:
        self._token_lists = token_lists

        self.get_all_json = async_to_raw_response_wrapper(
            token_lists.get_all_json,
        )


class TokenListsResourceWithStreamingResponse:
    def __init__(self, token_lists: TokenListsResource) -> None:
        self._token_lists = token_lists

        self.get_all_json = to_streamed_response_wrapper(
            token_lists.get_all_json,
        )


class AsyncTokenListsResourceWithStreamingResponse:
    def __init__(self, token_lists: AsyncTokenListsResource) -> None:
        self._token_lists = token_lists

        self.get_all_json = async_to_streamed_response_wrapper(
            token_lists.get_all_json,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from .exchanges import (
    ExchangesResource,
    AsyncExchangesResource,
    ExchangesResourceWithRawResponse,
    AsyncExchangesResourceWithRawResponse,
    ExchangesResourceWithStreamingResponse,
    AsyncExchangesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.derivative_get_response import DerivativeGetResponse

__all__ = ["DerivativesResource", "AsyncDerivativesResource"]


class DerivativesResource(SyncAPIResource):
    @cached_property
    def exchanges(self) -> ExchangesResource:
        return ExchangesResource(self._client)

    @cached_property
    def with_raw_response(self) -> DerivativesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return DerivativesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DerivativesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return DerivativesResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DerivativeGetResponse:
        """
        This endpoint allows you to **query all the tickers from derivatives exchanges
        on CoinGecko**
        """
        return self._get(
            "/derivatives",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DerivativeGetResponse,
        )


class AsyncDerivativesResource(AsyncAPIResource):
    @cached_property
    def exchanges(self) -> AsyncExchangesResource:
        return AsyncExchangesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDerivativesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDerivativesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDerivativesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncDerivativesResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DerivativeGetResponse:
        """
        This endpoint allows you to **query all the tickers from derivatives exchanges
        on CoinGecko**
        """
        return await self._get(
            "/derivatives",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DerivativeGetResponse,
        )


class DerivativesResourceWithRawResponse:
    def __init__(self, derivatives: DerivativesResource) -> None:
        self._derivatives = derivatives

        self.get = to_raw_response_wrapper(
            derivatives.get,
        )

    @cached_property
    def exchanges(self) -> ExchangesResourceWithRawResponse:
        return ExchangesResourceWithRawResponse(self._derivatives.exchanges)


class AsyncDerivativesResourceWithRawResponse:
    def __init__(self, derivatives: AsyncDerivativesResource) -> None:
        self._derivatives = derivatives

        self.get = async_to_raw_response_wrapper(
            derivatives.get,
        )

    @cached_property
    def exchanges(self) -> AsyncExchangesResourceWithRawResponse:
        return AsyncExchangesResourceWithRawResponse(self._derivatives.exchanges)


class DerivativesResourceWithStreamingResponse:
    def __init__(self, derivatives: DerivativesResource) -> None:
        self._derivatives = derivatives

        self.get = to_streamed_response_wrapper(
            derivatives.get,
        )

    @cached_property
    def exchanges(self) -> ExchangesResourceWithStreamingResponse:
        return ExchangesResourceWithStreamingResponse(self._derivatives.exchanges)


class AsyncDerivativesResourceWithStreamingResponse:
    def __init__(self, derivatives: AsyncDerivativesResource) -> None:
        self._derivatives = derivatives

        self.get = async_to_streamed_response_wrapper(
            derivatives.get,
        )

    @cached_property
    def exchanges(self) -> AsyncExchangesResourceWithStreamingResponse:
        return AsyncExchangesResourceWithStreamingResponse(self._derivatives.exchanges)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .key import (
    KeyResource,
    AsyncKeyResource,
    KeyResourceWithRawResponse,
    AsyncKeyResourceWithRawResponse,
    KeyResourceWithStreamingResponse,
    AsyncKeyResourceWithStreamingResponse,
)
from .nfts import (
    NFTsResource,
    AsyncNFTsResource,
    NFTsResourceWithRawResponse,
    AsyncNFTsResourceWithRawResponse,
    NFTsResourceWithStreamingResponse,
    AsyncNFTsResourceWithStreamingResponse,
)
from .ping import (
    PingResource,
    AsyncPingResource,
    PingResourceWithRawResponse,
    AsyncPingResourceWithRawResponse,
    PingResourceWithStreamingResponse,
    AsyncPingResourceWithStreamingResponse,
)
from .coins import (
    CoinsResource,
    AsyncCoinsResource,
    CoinsResourceWithRawResponse,
    AsyncCoinsResourceWithRawResponse,
    CoinsResourceWithStreamingResponse,
    AsyncCoinsResourceWithStreamingResponse,
)
from .search import (
    SearchResource,
    AsyncSearchResource,
    SearchResourceWithRawResponse,
    AsyncSearchResourceWithRawResponse,
    SearchResourceWithStreamingResponse,
    AsyncSearchResourceWithStreamingResponse,
)
from .simple import (
    SimpleResource,
    AsyncSimpleResource,
    SimpleResourceWithRawResponse,
    AsyncSimpleResourceWithRawResponse,
    SimpleResourceWithStreamingResponse,
    AsyncSimpleResourceWithStreamingResponse,
)
from .global_ import (
    GlobalResource,
    AsyncGlobalResource,
    GlobalResourceWithRawResponse,
    AsyncGlobalResourceWithRawResponse,
    GlobalResourceWithStreamingResponse,
    AsyncGlobalResourceWithStreamingResponse,
)
from .onchain import (
    OnchainResource,
    AsyncOnchainResource,
    OnchainResourceWithRawResponse,
    AsyncOnchainResourceWithRawResponse,
    OnchainResourceWithStreamingResponse,
    AsyncOnchainResourceWithStreamingResponse,
)
from .companies import (
    CompaniesResource,
    AsyncCompaniesResource,
    CompaniesResourceWithRawResponse,
    AsyncCompaniesResourceWithRawResponse,
    CompaniesResourceWithStreamingResponse,
    AsyncCompaniesResourceWithStreamingResponse,
)
from .exchanges import (
    ExchangesResource,
    AsyncExchangesResource,
    ExchangesResourceWithRawResponse,
    AsyncExchangesResourceWithRawResponse,
    ExchangesResourceWithStreamingResponse,
    AsyncExchangesResourceWithStreamingResponse,
)
from .derivatives import (
    DerivativesResource,
    AsyncDerivativesResource,
    DerivativesResourceWithRawResponse,
    AsyncDerivativesResourceWithRawResponse,
    DerivativesResourceWithStreamingResponse,
    AsyncDerivativesResourceWithStreamingResponse,
)
from .token_lists import (
    TokenListsResource,
    AsyncTokenListsResource,
    TokenListsResourceWithRawResponse,
    AsyncTokenListsResourceWithRawResponse,
    TokenListsResourceWithStreamingResponse,
    AsyncTokenListsResourceWithStreamingResponse,
)
from .exchange_rates import (
    ExchangeRatesResource,
    AsyncExchangeRatesResource,
    ExchangeRatesResourceWithRawResponse,
    AsyncExchangeRatesResourceWithRawResponse,
    ExchangeRatesResourceWithStreamingResponse,
    AsyncExchangeRatesResourceWithStreamingResponse,
)
from .asset_platforms import (
    AssetPlatformsResource,
    AsyncAssetPlatformsResource,
    AssetPlatformsResourceWithRawResponse,
    AsyncAssetPlatformsResourceWithRawResponse,
    AssetPlatformsResourceWithStreamingResponse,
    AsyncAssetPlatformsResourceWithStreamingResponse,
)

__all__ = [
    "AssetPlatformsResource",
    "AsyncAssetPlatformsResource",
    "AssetPlatformsResourceWithRawResponse",
    "AsyncAssetPlatformsResourceWithRawResponse",
    "AssetPlatformsResourceWithStreamingResponse",
    "AsyncAssetPlatformsResourceWithStreamingResponse",
    "CoinsResource",
    "AsyncCoinsResource",
    "CoinsResourceWithRawResponse",
    "AsyncCoinsResourceWithRawResponse",
    "CoinsResourceWithStreamingResponse",
    "AsyncCoinsResourceWithStreamingResponse",
    "CompaniesResource",
    "AsyncCompaniesResource",
    "CompaniesResourceWithRawResponse",
    "AsyncCompaniesResourceWithRawResponse",
    "CompaniesResourceWithStreamingResponse",
    "AsyncCompaniesResourceWithStreamingResponse",
    "DerivativesResource",
    "AsyncDerivativesResource",
    "DerivativesResourceWithRawResponse",
    "AsyncDerivativesResourceWithRawResponse",
    "DerivativesResourceWithStreamingResponse",
    "AsyncDerivativesResourceWithStreamingResponse",
    "ExchangeRatesResource",
    "AsyncExchangeRatesResource",
    "ExchangeRatesResourceWithRawResponse",
    "AsyncExchangeRatesResourceWithRawResponse",
    "ExchangeRatesResourceWithStreamingResponse",
    "AsyncExchangeRatesResourceWithStreamingResponse",
    "ExchangesResource",
    "AsyncExchangesResource",
    "ExchangesResourceWithRawResponse",
    "AsyncExchangesResourceWithRawResponse",
    "ExchangesResourceWithStreamingResponse",
    "AsyncExchangesResourceWithStreamingResponse",
    "GlobalResource",
    "AsyncGlobalResource",
    "GlobalResourceWithRawResponse",
    "AsyncGlobalResourceWithRawResponse",
    "GlobalResourceWithStreamingResponse",
    "AsyncGlobalResourceWithStreamingResponse",
    "KeyResource",
    "AsyncKeyResource",
    "KeyResourceWithRawResponse",
    "AsyncKeyResourceWithRawResponse",
    "KeyResourceWithStreamingResponse",
    "AsyncKeyResourceWithStreamingResponse",
    "NFTsResource",
    "AsyncNFTsResource",
    "NFTsResourceWithRawResponse",
    "AsyncNFTsResourceWithRawResponse",
    "NFTsResourceWithStreamingResponse",
    "AsyncNFTsResourceWithStreamingResponse",
    "OnchainResource",
    "AsyncOnchainResource",
    "OnchainResourceWithRawResponse",
    "AsyncOnchainResourceWithRawResponse",
    "OnchainResourceWithStreamingResponse",
    "AsyncOnchainResourceWithStreamingResponse",
    "PingResource",
    "AsyncPingResource",
    "PingResourceWithRawResponse",
    "AsyncPingResourceWithRawResponse",
    "PingResourceWithStreamingResponse",
    "AsyncPingResourceWithStreamingResponse",
    "SearchResource",
    "AsyncSearchResource",
    "SearchResourceWithRawResponse",
    "AsyncSearchResourceWithRawResponse",
    "SearchResourceWithStreamingResponse",
    "AsyncSearchResourceWithStreamingResponse",
    "SimpleResource",
    "AsyncSimpleResource",
    "SimpleResourceWithRawResponse",
    "AsyncSimpleResourceWithRawResponse",
    "SimpleResourceWithStreamingResponse",
    "AsyncSimpleResourceWithStreamingResponse",
    "TokenListsResource",
    "AsyncTokenListsResource",
    "TokenListsResourceWithRawResponse",
    "AsyncTokenListsResourceWithRawResponse",
    "TokenListsResourceWithStreamingResponse",
    "AsyncTokenListsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.onchain.networks.tokens import pool_get_params
from .....types.onchain.networks.tokens.pool_get_response import PoolGetResponse

__all__ = ["PoolsResource", "AsyncPoolsResource"]


class PoolsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PoolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return PoolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PoolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return PoolsResourceWithStreamingResponse(self)

    def get(
        self,
        token_address: str,
        *,
        network: str,
        include: str | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort: Literal["h24_volume_usd_liquidity_desc", "h24_tx_count_desc", "h24_volume_usd_desc"]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PoolGetResponse:
        """
        This endpoint allows you to **query top pools based on the provided token
        contract address on a network**

        Args:
          include: attributes to include, comma-separated if more than one to include Available
              values: `base_token`, `quote_token`, `dex`

          page: page through results Default value: 1

          sort: sort the pools by field Default value: h24_volume_usd_liquidity_desc

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not token_address:
            raise ValueError(f"Expected a non-empty value for `token_address` but received {token_address!r}")
        return self._get(
            f"/onchain/networks/{network}/tokens/{token_address}/pools",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include": include,
                        "page": page,
                        "sort": sort,
                    },
                    pool_get_params.PoolGetParams,
                ),
            ),
            cast_to=PoolGetResponse,
        )


class AsyncPoolsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPoolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPoolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPoolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncPoolsResourceWithStreamingResponse(self)

    async def get(
        self,
        token_address: str,
        *,
        network: str,
        include: str | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort: Literal["h24_volume_usd_liquidity_desc", "h24_tx_count_desc", "h24_volume_usd_desc"]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PoolGetResponse:
        """
        This endpoint allows you to **query top pools based on the provided token
        contract address on a network**

        Args:
          include: attributes to include, comma-separated if more than one to include Available
              values: `base_token`, `quote_token`, `dex`

          page: page through results Default value: 1

          sort: sort the pools by field Default value: h24_volume_usd_liquidity_desc

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not token_address:
            raise ValueError(f"Expected a non-empty value for `token_address` but received {token_address!r}")
        return await self._get(
            f"/onchain/networks/{network}/tokens/{token_address}/pools",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include": include,
                        "page": page,
                        "sort": sort,
                    },
                    pool_get_params.PoolGetParams,
                ),
            ),
            cast_to=PoolGetResponse,
        )


class PoolsResourceWithRawResponse:
    def __init__(self, pools: PoolsResource) -> None:
        self._pools = pools

        self.get = to_raw_response_wrapper(
            pools.get,
        )


class AsyncPoolsResourceWithRawResponse:
    def __init__(self, pools: AsyncPoolsResource) -> None:
        self._pools = pools

        self.get = async_to_raw_response_wrapper(
            pools.get,
        )


class PoolsResourceWithStreamingResponse:
    def __init__(self, pools: PoolsResource) -> None:
        self._pools = pools

        self.get = to_streamed_response_wrapper(
            pools.get,
        )


class AsyncPoolsResourceWithStreamingResponse:
    def __init__(self, pools: AsyncPoolsResource) -> None:
        self._pools = pools

        self.get = async_to_streamed_response_wrapper(
            pools.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Dict, Union, Mapping, cast
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import key, ping, token_lists, exchange_rates, asset_platforms
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.nfts import nfts
from .resources.coins import coins
from .resources.search import search
from .resources.simple import simple
from .resources.global_ import global_
from .resources.onchain import onchain
from .resources.companies import companies
from .resources.exchanges import exchanges
from .resources.derivatives import derivatives

__all__ = [
    "ENVIRONMENTS",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Coingecko",
    "AsyncCoingecko",
    "Client",
    "AsyncClient",
]

ENVIRONMENTS: Dict[str, str] = {
    "pro": "https://pro-api.coingecko.com/api/v3",
    "demo": "https://api.coingecko.com/api/v3",
}


class Coingecko(SyncAPIClient):
    asset_platforms: asset_platforms.AssetPlatformsResource
    coins: coins.CoinsResource
    companies: companies.CompaniesResource
    derivatives: derivatives.DerivativesResource
    exchange_rates: exchange_rates.ExchangeRatesResource
    exchanges: exchanges.ExchangesResource
    global_: global_.GlobalResource
    key: key.KeyResource
    nfts: nfts.NFTsResource
    onchain: onchain.OnchainResource
    ping: ping.PingResource
    search: search.SearchResource
    simple: simple.SimpleResource
    token_lists: token_lists.TokenListsResource
    with_raw_response: CoingeckoWithRawResponse
    with_streaming_response: CoingeckoWithStreamedResponse

    # client options
    pro_api_key: str | None
    demo_api_key: str | None

    _environment: Literal["pro", "demo"] | NotGiven

    def __init__(
        self,
        *,
        pro_api_key: str | None = None,
        demo_api_key: str | None = None,
        environment: Literal["pro", "demo"] | NotGiven = NOT_GIVEN,
        base_url: str | httpx.URL | None | NotGiven = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Coingecko client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `pro_api_key` from `COINGECKO_PRO_API_KEY`
        - `demo_api_key` from `COINGECKO_DEMO_API_KEY`
        """
        if pro_api_key is None:
            pro_api_key = os.environ.get("COINGECKO_PRO_API_KEY")
        self.pro_api_key = pro_api_key

        if demo_api_key is None:
            demo_api_key = os.environ.get("COINGECKO_DEMO_API_KEY")
        self.demo_api_key = demo_api_key

        self._environment = environment

        base_url_env = os.environ.get("COINGECKO_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `COINGECKO_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "pro"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.asset_platforms = asset_platforms.AssetPlatformsResource(self)
        self.coins = coins.CoinsResource(self)
        self.companies = companies.CompaniesResource(self)
        self.derivatives = derivatives.DerivativesResource(self)
        self.exchange_rates = exchange_rates.ExchangeRatesResource(self)
        self.exchanges = exchanges.ExchangesResource(self)
        self.global_ = global_.GlobalResource(self)
        self.key = key.KeyResource(self)
        self.nfts = nfts.NFTsResource(self)
        self.onchain = onchain.OnchainResource(self)
        self.ping = ping.PingResource(self)
        self.search = search.SearchResource(self)
        self.simple = simple.SimpleResource(self)
        self.token_lists = token_lists.TokenListsResource(self)
        self.with_raw_response = CoingeckoWithRawResponse(self)
        self.with_streaming_response = CoingeckoWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        return {**self._pro_key_auth, **self._demo_key_auth}

    @property
    def _pro_key_auth(self) -> dict[str, str]:
        pro_api_key = self.pro_api_key
        if pro_api_key is None:
            return {}
        return {"x-cg-pro-api-key": pro_api_key}

    @property
    def _demo_key_auth(self) -> dict[str, str]:
        demo_api_key = self.demo_api_key
        if demo_api_key is None:
            return {}
        return {"x-cg-demo-api-key": demo_api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if self.pro_api_key and headers.get("x-cg-pro-api-key"):
            return
        if isinstance(custom_headers.get("x-cg-pro-api-key"), Omit):
            return

        if self.demo_api_key and headers.get("x-cg-demo-api-key"):
            return
        if isinstance(custom_headers.get("x-cg-demo-api-key"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected either pro_api_key or demo_api_key to be set. Or for one of the `x-cg-pro-api-key` or `x-cg-demo-api-key` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        pro_api_key: str | None = None,
        demo_api_key: str | None = None,
        environment: Literal["pro", "demo"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            pro_api_key=pro_api_key or self.pro_api_key,
            demo_api_key=demo_api_key or self.demo_api_key,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncCoingecko(AsyncAPIClient):
    asset_platforms: asset_platforms.AsyncAssetPlatformsResource
    coins: coins.AsyncCoinsResource
    companies: companies.AsyncCompaniesResource
    derivatives: derivatives.AsyncDerivativesResource
    exchange_rates: exchange_rates.AsyncExchangeRatesResource
    exchanges: exchanges.AsyncExchangesResource
    global_: global_.AsyncGlobalResource
    key: key.AsyncKeyResource
    nfts: nfts.AsyncNFTsResource
    onchain: onchain.AsyncOnchainResource
    ping: ping.AsyncPingResource
    search: search.AsyncSearchResource
    simple: simple.AsyncSimpleResource
    token_lists: token_lists.AsyncTokenListsResource
    with_raw_response: AsyncCoingeckoWithRawResponse
    with_streaming_response: AsyncCoingeckoWithStreamedResponse

    # client options
    pro_api_key: str | None
    demo_api_key: str | None

    _environment: Literal["pro", "demo"] | NotGiven

    def __init__(
        self,
        *,
        pro_api_key: str | None = None,
        demo_api_key: str | None = None,
        environment: Literal["pro", "demo"] | NotGiven = NOT_GIVEN,
        base_url: str | httpx.URL | None | NotGiven = NOT_GIVEN,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncCoingecko client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `pro_api_key` from `COINGECKO_PRO_API_KEY`
        - `demo_api_key` from `COINGECKO_DEMO_API_KEY`
        """
        if pro_api_key is None:
            pro_api_key = os.environ.get("COINGECKO_PRO_API_KEY")
        self.pro_api_key = pro_api_key

        if demo_api_key is None:
            demo_api_key = os.environ.get("COINGECKO_DEMO_API_KEY")
        self.demo_api_key = demo_api_key

        self._environment = environment

        base_url_env = os.environ.get("COINGECKO_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `COINGECKO_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "pro"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.asset_platforms = asset_platforms.AsyncAssetPlatformsResource(self)
        self.coins = coins.AsyncCoinsResource(self)
        self.companies = companies.AsyncCompaniesResource(self)
        self.derivatives = derivatives.AsyncDerivativesResource(self)
        self.exchange_rates = exchange_rates.AsyncExchangeRatesResource(self)
        self.exchanges = exchanges.AsyncExchangesResource(self)
        self.global_ = global_.AsyncGlobalResource(self)
        self.key = key.AsyncKeyResource(self)
        self.nfts = nfts.AsyncNFTsResource(self)
        self.onchain = onchain.AsyncOnchainResource(self)
        self.ping = ping.AsyncPingResource(self)
        self.search = search.AsyncSearchResource(self)
        self.simple = simple.AsyncSimpleResource(self)
        self.token_lists = token_lists.AsyncTokenListsResource(self)
        self.with_raw_response = AsyncCoingeckoWithRawResponse(self)
        self.with_streaming_response = AsyncCoingeckoWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        return {**self._pro_key_auth, **self._demo_key_auth}

    @property
    def _pro_key_auth(self) -> dict[str, str]:
        pro_api_key = self.pro_api_key
        if pro_api_key is None:
            return {}
        return {"x-cg-pro-api-key": pro_api_key}

    @property
    def _demo_key_auth(self) -> dict[str, str]:
        demo_api_key = self.demo_api_key
        if demo_api_key is None:
            return {}
        return {"x-cg-demo-api-key": demo_api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if self.pro_api_key and headers.get("x-cg-pro-api-key"):
            return
        if isinstance(custom_headers.get("x-cg-pro-api-key"), Omit):
            return

        if self.demo_api_key and headers.get("x-cg-demo-api-key"):
            return
        if isinstance(custom_headers.get("x-cg-demo-api-key"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected either pro_api_key or demo_api_key to be set. Or for one of the `x-cg-pro-api-key` or `x-cg-demo-api-key` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        pro_api_key: str | None = None,
        demo_api_key: str | None = None,
        environment: Literal["pro", "demo"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            pro_api_key=pro_api_key or self.pro_api_key,
            demo_api_key=demo_api_key or self.demo_api_key,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class CoingeckoWithRawResponse:
    def __init__(self, client: Coingecko) -> None:
        self.asset_platforms = asset_platforms.AssetPlatformsResourceWithRawResponse(client.asset_platforms)
        self.coins = coins.CoinsResourceWithRawResponse(client.coins)
        self.companies = companies.CompaniesResourceWithRawResponse(client.companies)
        self.derivatives = derivatives.DerivativesResourceWithRawResponse(client.derivatives)
        self.exchange_rates = exchange_rates.ExchangeRatesResourceWithRawResponse(client.exchange_rates)
        self.exchanges = exchanges.ExchangesResourceWithRawResponse(client.exchanges)
        self.global_ = global_.GlobalResourceWithRawResponse(client.global_)
        self.key = key.KeyResourceWithRawResponse(client.key)
        self.nfts = nfts.NFTsResourceWithRawResponse(client.nfts)
        self.onchain = onchain.OnchainResourceWithRawResponse(client.onchain)
        self.ping = ping.PingResourceWithRawResponse(client.ping)
        self.search = search.SearchResourceWithRawResponse(client.search)
        self.simple = simple.SimpleResourceWithRawResponse(client.simple)
        self.token_lists = token_lists.TokenListsResourceWithRawResponse(client.token_lists)


class AsyncCoingeckoWithRawResponse:
    def __init__(self, client: AsyncCoingecko) -> None:
        self.asset_platforms = asset_platforms.AsyncAssetPlatformsResourceWithRawResponse(client.asset_platforms)
        self.coins = coins.AsyncCoinsResourceWithRawResponse(client.coins)
        self.companies = companies.AsyncCompaniesResourceWithRawResponse(client.companies)
        self.derivatives = derivatives.AsyncDerivativesResourceWithRawResponse(client.derivatives)
        self.exchange_rates = exchange_rates.AsyncExchangeRatesResourceWithRawResponse(client.exchange_rates)
        self.exchanges = exchanges.AsyncExchangesResourceWithRawResponse(client.exchanges)
        self.global_ = global_.AsyncGlobalResourceWithRawResponse(client.global_)
        self.key = key.AsyncKeyResourceWithRawResponse(client.key)
        self.nfts = nfts.AsyncNFTsResourceWithRawResponse(client.nfts)
        self.onchain = onchain.AsyncOnchainResourceWithRawResponse(client.onchain)
        self.ping = ping.AsyncPingResourceWithRawResponse(client.ping)
        self.search = search.AsyncSearchResourceWithRawResponse(client.search)
        self.simple = simple.AsyncSimpleResourceWithRawResponse(client.simple)
        self.token_lists = token_lists.AsyncTokenListsResourceWithRawResponse(client.token_lists)


class CoingeckoWithStreamedResponse:
    def __init__(self, client: Coingecko) -> None:
        self.asset_platforms = asset_platforms.AssetPlatformsResourceWithStreamingResponse(client.asset_platforms)
        self.coins = coins.CoinsResourceWithStreamingResponse(client.coins)
        self.companies = companies.CompaniesResourceWithStreamingResponse(client.companies)
        self.derivatives = derivatives.DerivativesResourceWithStreamingResponse(client.derivatives)
        self.exchange_rates = exchange_rates.ExchangeRatesResourceWithStreamingResponse(client.exchange_rates)
        self.exchanges = exchanges.ExchangesResourceWithStreamingResponse(client.exchanges)
        self.global_ = global_.GlobalResourceWithStreamingResponse(client.global_)
        self.key = key.KeyResourceWithStreamingResponse(client.key)
        self.nfts = nfts.NFTsResourceWithStreamingResponse(client.nfts)
        self.onchain = onchain.OnchainResourceWithStreamingResponse(client.onchain)
        self.ping = ping.PingResourceWithStreamingResponse(client.ping)
        self.search = search.SearchResourceWithStreamingResponse(client.search)
        self.simple = simple.SimpleResourceWithStreamingResponse(client.simple)
        self.token_lists = token_lists.TokenListsResourceWithStreamingResponse(client.token_lists)


class AsyncCoingeckoWithStreamedResponse:
    def __init__(self, client: AsyncCoingecko) -> None:
        self.asset_platforms = asset_platforms.AsyncAssetPlatformsResourceWithStreamingResponse(client.asset_platforms)
        self.coins = coins.AsyncCoinsResourceWithStreamingResponse(client.coins)
        self.companies = companies.AsyncCompaniesResourceWithStreamingResponse(client.companies)
        self.derivatives = derivatives.AsyncDerivativesResourceWithStreamingResponse(client.derivatives)
        self.exchange_rates = exchange_rates.AsyncExchangeRatesResourceWithStreamingResponse(client.exchange_rates)
        self.exchanges = exchanges.AsyncExchangesResourceWithStreamingResponse(client.exchanges)
        self.global_ = global_.AsyncGlobalResourceWithStreamingResponse(client.global_)
        self.key = key.AsyncKeyResourceWithStreamingResponse(client.key)
        self.nfts = nfts.AsyncNFTsResourceWithStreamingResponse(client.nfts)
        self.onchain = onchain.AsyncOnchainResourceWithStreamingResponse(client.onchain)
        self.ping = ping.AsyncPingResourceWithStreamingResponse(client.ping)
        self.search = search.AsyncSearchResourceWithStreamingResponse(client.search)
        self.simple = simple.AsyncSimpleResourceWithStreamingResponse(client.simple)
        self.token_lists = token_lists.AsyncTokenListsResourceWithStreamingResponse(client.token_lists)


Client = Coingecko

AsyncClient = AsyncCoingecko



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.derivatives import exchange_get_params, exchange_get_id_params
from ...types.derivatives.exchange_get_response import ExchangeGetResponse
from ...types.derivatives.exchange_get_id_response import ExchangeGetIDResponse
from ...types.derivatives.exchange_get_list_response import ExchangeGetListResponse

__all__ = ["ExchangesResource", "AsyncExchangesResource"]


class ExchangesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ExchangesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return ExchangesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExchangesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return ExchangesResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        order: Literal[
            "name_asc",
            "name_desc",
            "open_interest_btc_asc",
            "open_interest_btc_desc",
            "trade_volume_24h_btc_asc",
            "trade_volume_24h_btc_desc",
        ]
        | NotGiven = NOT_GIVEN,
        page: float | NotGiven = NOT_GIVEN,
        per_page: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetResponse:
        """
        This endpoint allows you to **query all the derivatives exchanges with related
        data (ID, name, open interest, ...) on CoinGecko**

        Args:
          order: use this to sort the order of responses, default: open_interest_btc_desc

          page: page through results, default: 1

          per_page: total results per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/derivatives/exchanges",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    exchange_get_params.ExchangeGetParams,
                ),
            ),
            cast_to=ExchangeGetResponse,
        )

    def get_id(
        self,
        id: str,
        *,
        include_tickers: Literal["all", "unexpired"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetIDResponse:
        """
        This endpoint allows you to **query the derivatives exchange’s related data (ID,
        name, open interest, ...) based on the exchanges’ ID**

        Args:
          include_tickers: include tickers data

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/derivatives/exchanges/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include_tickers": include_tickers}, exchange_get_id_params.ExchangeGetIDParams),
            ),
            cast_to=ExchangeGetIDResponse,
        )

    def get_list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetListResponse:
        """
        This endpoint allows you to **query all the derivatives exchanges with ID and
        name on CoinGecko**
        """
        return self._get(
            "/derivatives/exchanges/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExchangeGetListResponse,
        )


class AsyncExchangesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncExchangesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExchangesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExchangesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncExchangesResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        order: Literal[
            "name_asc",
            "name_desc",
            "open_interest_btc_asc",
            "open_interest_btc_desc",
            "trade_volume_24h_btc_asc",
            "trade_volume_24h_btc_desc",
        ]
        | NotGiven = NOT_GIVEN,
        page: float | NotGiven = NOT_GIVEN,
        per_page: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetResponse:
        """
        This endpoint allows you to **query all the derivatives exchanges with related
        data (ID, name, open interest, ...) on CoinGecko**

        Args:
          order: use this to sort the order of responses, default: open_interest_btc_desc

          page: page through results, default: 1

          per_page: total results per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/derivatives/exchanges",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    exchange_get_params.ExchangeGetParams,
                ),
            ),
            cast_to=ExchangeGetResponse,
        )

    async def get_id(
        self,
        id: str,
        *,
        include_tickers: Literal["all", "unexpired"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetIDResponse:
        """
        This endpoint allows you to **query the derivatives exchange’s related data (ID,
        name, open interest, ...) based on the exchanges’ ID**

        Args:
          include_tickers: include tickers data

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/derivatives/exchanges/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include_tickers": include_tickers}, exchange_get_id_params.ExchangeGetIDParams
                ),
            ),
            cast_to=ExchangeGetIDResponse,
        )

    async def get_list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetListResponse:
        """
        This endpoint allows you to **query all the derivatives exchanges with ID and
        name on CoinGecko**
        """
        return await self._get(
            "/derivatives/exchanges/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExchangeGetListResponse,
        )


class ExchangesResourceWithRawResponse:
    def __init__(self, exchanges: ExchangesResource) -> None:
        self._exchanges = exchanges

        self.get = to_raw_response_wrapper(
            exchanges.get,
        )
        self.get_id = to_raw_response_wrapper(
            exchanges.get_id,
        )
        self.get_list = to_raw_response_wrapper(
            exchanges.get_list,
        )


class AsyncExchangesResourceWithRawResponse:
    def __init__(self, exchanges: AsyncExchangesResource) -> None:
        self._exchanges = exchanges

        self.get = async_to_raw_response_wrapper(
            exchanges.get,
        )
        self.get_id = async_to_raw_response_wrapper(
            exchanges.get_id,
        )
        self.get_list = async_to_raw_response_wrapper(
            exchanges.get_list,
        )


class ExchangesResourceWithStreamingResponse:
    def __init__(self, exchanges: ExchangesResource) -> None:
        self._exchanges = exchanges

        self.get = to_streamed_response_wrapper(
            exchanges.get,
        )
        self.get_id = to_streamed_response_wrapper(
            exchanges.get_id,
        )
        self.get_list = to_streamed_response_wrapper(
            exchanges.get_list,
        )


class AsyncExchangesResourceWithStreamingResponse:
    def __init__(self, exchanges: AsyncExchangesResource) -> None:
        self._exchanges = exchanges

        self.get = async_to_streamed_response_wrapper(
            exchanges.get,
        )
        self.get_id = async_to_streamed_response_wrapper(
            exchanges.get_id,
        )
        self.get_list = async_to_streamed_response_wrapper(
            exchanges.get_list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.onchain.networks import trending_pool_get_params, trending_pool_get_network_params
from ....types.onchain.networks.trending_pool_get_response import TrendingPoolGetResponse
from ....types.onchain.networks.trending_pool_get_network_response import TrendingPoolGetNetworkResponse

__all__ = ["TrendingPoolsResource", "AsyncTrendingPoolsResource"]


class TrendingPoolsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TrendingPoolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return TrendingPoolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TrendingPoolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return TrendingPoolsResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        duration: Literal["5m", "1h", "6h", "24h"] | NotGiven = NOT_GIVEN,
        include: str | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TrendingPoolGetResponse:
        """
        This endpoint allows you to **query all the trending pools across all networks
        on GeckoTerminal**

        Args:
          duration: duration to sort trending list by Default value: 24h

          include: attributes to include, comma-separated if more than one to include Available
              values: `base_token`, `quote_token`, `dex`, `network`. Example: `base_token` or
              `base_token,dex`

          page: page through results Default value: 1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/onchain/networks/trending_pools",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "duration": duration,
                        "include": include,
                        "page": page,
                    },
                    trending_pool_get_params.TrendingPoolGetParams,
                ),
            ),
            cast_to=TrendingPoolGetResponse,
        )

    def get_network(
        self,
        network: str,
        *,
        duration: Literal["5m", "1h", "6h", "24h"] | NotGiven = NOT_GIVEN,
        include: str | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TrendingPoolGetNetworkResponse:
        """
        This endpoint allows you to **query the trending pools based on the provided
        network**

        Args:
          duration: duration to sort trending list by Default value: 24h

          include: attributes to include, comma-separated if more than one to include Available
              values: `base_token`, `quote_token`, `dex`

          page: page through results Default value: 1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        return self._get(
            f"/onchain/networks/{network}/trending_pools",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "duration": duration,
                        "include": include,
                        "page": page,
                    },
                    trending_pool_get_network_params.TrendingPoolGetNetworkParams,
                ),
            ),
            cast_to=TrendingPoolGetNetworkResponse,
        )


class AsyncTrendingPoolsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTrendingPoolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTrendingPoolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTrendingPoolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncTrendingPoolsResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        duration: Literal["5m", "1h", "6h", "24h"] | NotGiven = NOT_GIVEN,
        include: str | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TrendingPoolGetResponse:
        """
        This endpoint allows you to **query all the trending pools across all networks
        on GeckoTerminal**

        Args:
          duration: duration to sort trending list by Default value: 24h

          include: attributes to include, comma-separated if more than one to include Available
              values: `base_token`, `quote_token`, `dex`, `network`. Example: `base_token` or
              `base_token,dex`

          page: page through results Default value: 1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/onchain/networks/trending_pools",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "duration": duration,
                        "include": include,
                        "page": page,
                    },
                    trending_pool_get_params.TrendingPoolGetParams,
                ),
            ),
            cast_to=TrendingPoolGetResponse,
        )

    async def get_network(
        self,
        network: str,
        *,
        duration: Literal["5m", "1h", "6h", "24h"] | NotGiven = NOT_GIVEN,
        include: str | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TrendingPoolGetNetworkResponse:
        """
        This endpoint allows you to **query the trending pools based on the provided
        network**

        Args:
          duration: duration to sort trending list by Default value: 24h

          include: attributes to include, comma-separated if more than one to include Available
              values: `base_token`, `quote_token`, `dex`

          page: page through results Default value: 1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        return await self._get(
            f"/onchain/networks/{network}/trending_pools",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "duration": duration,
                        "include": include,
                        "page": page,
                    },
                    trending_pool_get_network_params.TrendingPoolGetNetworkParams,
                ),
            ),
            cast_to=TrendingPoolGetNetworkResponse,
        )


class TrendingPoolsResourceWithRawResponse:
    def __init__(self, trending_pools: TrendingPoolsResource) -> None:
        self._trending_pools = trending_pools

        self.get = to_raw_response_wrapper(
            trending_pools.get,
        )
        self.get_network = to_raw_response_wrapper(
            trending_pools.get_network,
        )


class AsyncTrendingPoolsResourceWithRawResponse:
    def __init__(self, trending_pools: AsyncTrendingPoolsResource) -> None:
        self._trending_pools = trending_pools

        self.get = async_to_raw_response_wrapper(
            trending_pools.get,
        )
        self.get_network = async_to_raw_response_wrapper(
            trending_pools.get_network,
        )


class TrendingPoolsResourceWithStreamingResponse:
    def __init__(self, trending_pools: TrendingPoolsResource) -> None:
        self._trending_pools = trending_pools

        self.get = to_streamed_response_wrapper(
            trending_pools.get,
        )
        self.get_network = to_streamed_response_wrapper(
            trending_pools.get_network,
        )


class AsyncTrendingPoolsResourceWithStreamingResponse:
    def __init__(self, trending_pools: AsyncTrendingPoolsResource) -> None:
        self._trending_pools = trending_pools

        self.get = async_to_streamed_response_wrapper(
            trending_pools.get,
        )
        self.get_network = async_to_streamed_response_wrapper(
            trending_pools.get_network,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["MultiGetAddressesParams"]


class MultiGetAddressesParams(TypedDict, total=False):
    network: Required[str]

    include: str
    """
    attributes to include, comma-separated if more than one to include Available
    values: `base_token`, `quote_token`, `dex`
    """

    include_volume_breakdown: bool
    """include volume breakdown, default: false"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .companies import (
    CompaniesResource,
    AsyncCompaniesResource,
    CompaniesResourceWithRawResponse,
    AsyncCompaniesResourceWithRawResponse,
    CompaniesResourceWithStreamingResponse,
    AsyncCompaniesResourceWithStreamingResponse,
)
from .public_treasury import (
    PublicTreasuryResource,
    AsyncPublicTreasuryResource,
    PublicTreasuryResourceWithRawResponse,
    AsyncPublicTreasuryResourceWithRawResponse,
    PublicTreasuryResourceWithStreamingResponse,
    AsyncPublicTreasuryResourceWithStreamingResponse,
)

__all__ = [
    "PublicTreasuryResource",
    "AsyncPublicTreasuryResource",
    "PublicTreasuryResourceWithRawResponse",
    "AsyncPublicTreasuryResourceWithRawResponse",
    "PublicTreasuryResourceWithStreamingResponse",
    "AsyncPublicTreasuryResourceWithStreamingResponse",
    "CompaniesResource",
    "AsyncCompaniesResource",
    "CompaniesResourceWithRawResponse",
    "AsyncCompaniesResourceWithRawResponse",
    "CompaniesResourceWithStreamingResponse",
    "AsyncCompaniesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .price_get_params import PriceGetParams as PriceGetParams
from .price_get_response import PriceGetResponse as PriceGetResponse
from .token_price_get_id_params import TokenPriceGetIDParams as TokenPriceGetIDParams
from .token_price_get_id_response import TokenPriceGetIDResponse as TokenPriceGetIDResponse
from .supported_vs_currency_get_response import SupportedVsCurrencyGetResponse as SupportedVsCurrencyGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = [
    "TrendingGetResponse",
    "Category",
    "CategoryData",
    "CategoryDataMarketCapChangePercentage24h",
    "Coin",
    "CoinData",
    "CoinDataPriceChangePercentage24h",
    "NFT",
    "NFTData",
]


class CategoryDataMarketCapChangePercentage24h(BaseModel):
    btc: Optional[float] = None

    usd: Optional[float] = None


class CategoryData(BaseModel):
    market_cap: Optional[float] = None
    """category market cap"""

    market_cap_btc: Optional[float] = None
    """category market cap in btc"""

    market_cap_change_percentage_24h: Optional[CategoryDataMarketCapChangePercentage24h] = None
    """category market cap change percentage in 24 hours"""

    sparkline: Optional[str] = None
    """category sparkline image url"""

    total_volume: Optional[float] = None
    """category total volume"""

    total_volume_btc: Optional[float] = None
    """category total volume in btc"""


class Category(BaseModel):
    id: Optional[float] = None

    coins_count: Optional[float] = None
    """category number of coins"""

    data: Optional[CategoryData] = None

    market_cap_1h_change: Optional[float] = None
    """category market cap 1 hour change"""

    name: Optional[str] = None
    """category name"""

    slug: Optional[str] = None
    """category web slug"""


class CoinDataPriceChangePercentage24h(BaseModel):
    btc: Optional[float] = None

    usd: Optional[float] = None


class CoinData(BaseModel):
    content: Optional[str] = None

    market_cap: Optional[str] = None
    """coin market cap in usd"""

    market_cap_btc: Optional[str] = None
    """coin market cap in btc"""

    price: Optional[float] = None
    """coin price in usd"""

    price_btc: Optional[str] = None
    """coin price in btc"""

    price_change_percentage_24h: Optional[CoinDataPriceChangePercentage24h] = None
    """coin price change percentage in 24 hours"""

    sparkline: Optional[str] = None
    """coin sparkline image url"""

    total_volume: Optional[str] = None
    """coin total volume in usd"""

    total_volume_btc: Optional[str] = None
    """coin total volume in btc"""


class Coin(BaseModel):
    id: Optional[str] = None
    """coin ID"""

    coin_id: Optional[float] = None

    data: Optional[CoinData] = None

    large: Optional[str] = None
    """coin large image url"""

    market_cap_rank: Optional[float] = None
    """coin market cap rank"""

    name: Optional[str] = None
    """coin name"""

    price_btc: Optional[float] = None
    """coin price in btc"""

    score: Optional[float] = None
    """coin sequence in the list"""

    slug: Optional[str] = None
    """coin web slug"""

    small: Optional[str] = None
    """coin small image url"""

    symbol: Optional[str] = None
    """coin symbol"""

    thumb: Optional[str] = None
    """coin thumb image url"""


class NFTData(BaseModel):
    content: Optional[str] = None

    floor_price: Optional[str] = None
    """NFT collection floor price"""

    floor_price_in_usd_24h_percentage_change: Optional[str] = None
    """NFT collection floor price in usd 24 hours percentage change"""

    h24_average_sale_price: Optional[str] = None
    """NFT collection 24 hours average sale price"""

    h24_volume: Optional[str] = None
    """NFT collection volume in 24 hours"""

    sparkline: Optional[str] = None
    """NFT collection sparkline image url"""


class NFT(BaseModel):
    id: Optional[str] = None
    """NFT collection ID"""

    data: Optional[NFTData] = None

    floor_price_24h_percentage_change: Optional[float] = None
    """NFT collection floor price 24 hours percentage change"""

    floor_price_in_native_currency: Optional[float] = None
    """NFT collection floor price in native currency"""

    name: Optional[str] = None
    """NFT collection name"""

    native_currency_symbol: Optional[str] = None
    """NFT collection native currency symbol"""

    nft_contract_id: Optional[float] = None

    symbol: Optional[str] = None
    """NFT collection symbol"""

    thumb: Optional[str] = None
    """NFT collection thumb image url"""


class TrendingGetResponse(BaseModel):
    categories: Optional[List[Category]] = None

    coins: Optional[List[Coin]] = None

    nfts: Optional[List[NFT]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = ["MarketGetResponse", "MarketGetResponseItem", "MarketGetResponseItemRoi"]


class MarketGetResponseItemRoi(BaseModel):
    currency: str
    """ROI currency"""

    percentage: float
    """ROI percentage"""

    times: float
    """ROI multiplier"""


class MarketGetResponseItem(BaseModel):
    id: Optional[str] = None
    """coin ID"""

    ath: Optional[float] = None
    """coin all time high (ATH) in currency"""

    ath_change_percentage: Optional[float] = None
    """coin all time high (ATH) change in percentage"""

    ath_date: Optional[datetime] = None
    """coin all time high (ATH) date"""

    atl: Optional[float] = None
    """coin all time low (atl) in currency"""

    atl_change_percentage: Optional[float] = None
    """coin all time low (atl) change in percentage"""

    atl_date: Optional[datetime] = None
    """coin all time low (atl) date"""

    circulating_supply: Optional[float] = None
    """coin circulating supply"""

    current_price: Optional[float] = None
    """coin current price in currency"""

    fully_diluted_valuation: Optional[float] = None
    """coin fully diluted valuation (fdv) in currency"""

    high_24h: Optional[float] = None
    """coin 24hr price high in currency"""

    image: Optional[str] = None
    """coin image url"""

    last_updated: Optional[datetime] = None
    """coin last updated timestamp"""

    low_24h: Optional[float] = None
    """coin 24hr price low in currency"""

    market_cap: Optional[float] = None
    """coin market cap in currency"""

    market_cap_change_24h: Optional[float] = None
    """coin 24hr market cap change in currency"""

    market_cap_change_percentage_24h: Optional[float] = None
    """coin 24hr market cap change in percentage"""

    market_cap_rank: Optional[float] = None
    """coin rank by market cap"""

    max_supply: Optional[float] = None
    """coin max supply"""

    name: Optional[str] = None
    """coin name"""

    price_change_24h: Optional[float] = None
    """coin 24hr price change in currency"""

    price_change_percentage_24h: Optional[float] = None
    """coin 24hr price change in percentage"""

    roi: Optional[MarketGetResponseItemRoi] = None
    """return on investment data"""

    symbol: Optional[str] = None
    """coin symbol"""

    total_supply: Optional[float] = None
    """coin total supply"""

    total_volume: Optional[float] = None
    """coin total trading volume in currency"""


MarketGetResponse: TypeAlias = List[MarketGetResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["InfoRecentlyUpdatedGetParams"]


class InfoRecentlyUpdatedGetParams(TypedDict, total=False):
    include: Literal["network"]
    """
    Attributes for related resources to include, which will be returned under the
    top-level 'included' key
    """

    network: str
    """
    filter tokens by provided network \\**refers to
    [/networks](/reference/networks-list)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .dexes import (
    DexesResource,
    AsyncDexesResource,
    DexesResourceWithRawResponse,
    AsyncDexesResourceWithRawResponse,
    DexesResourceWithStreamingResponse,
    AsyncDexesResourceWithStreamingResponse,
)
from .pools import (
    PoolsResource,
    AsyncPoolsResource,
    PoolsResourceWithRawResponse,
    AsyncPoolsResourceWithRawResponse,
    PoolsResourceWithStreamingResponse,
    AsyncPoolsResourceWithStreamingResponse,
)
from .tokens import (
    TokensResource,
    AsyncTokensResource,
    TokensResourceWithRawResponse,
    AsyncTokensResourceWithRawResponse,
    TokensResourceWithStreamingResponse,
    AsyncTokensResourceWithStreamingResponse,
)
from .networks import (
    NetworksResource,
    AsyncNetworksResource,
    NetworksResourceWithRawResponse,
    AsyncNetworksResourceWithRawResponse,
    NetworksResourceWithStreamingResponse,
    AsyncNetworksResourceWithStreamingResponse,
)
from .new_pools import (
    NewPoolsResource,
    AsyncNewPoolsResource,
    NewPoolsResourceWithRawResponse,
    AsyncNewPoolsResourceWithRawResponse,
    NewPoolsResourceWithStreamingResponse,
    AsyncNewPoolsResourceWithStreamingResponse,
)
from .trending_pools import (
    TrendingPoolsResource,
    AsyncTrendingPoolsResource,
    TrendingPoolsResourceWithRawResponse,
    AsyncTrendingPoolsResourceWithRawResponse,
    TrendingPoolsResourceWithStreamingResponse,
    AsyncTrendingPoolsResourceWithStreamingResponse,
)

__all__ = [
    "NewPoolsResource",
    "AsyncNewPoolsResource",
    "NewPoolsResourceWithRawResponse",
    "AsyncNewPoolsResourceWithRawResponse",
    "NewPoolsResourceWithStreamingResponse",
    "AsyncNewPoolsResourceWithStreamingResponse",
    "TrendingPoolsResource",
    "AsyncTrendingPoolsResource",
    "TrendingPoolsResourceWithRawResponse",
    "AsyncTrendingPoolsResourceWithRawResponse",
    "TrendingPoolsResourceWithStreamingResponse",
    "AsyncTrendingPoolsResourceWithStreamingResponse",
    "DexesResource",
    "AsyncDexesResource",
    "DexesResourceWithRawResponse",
    "AsyncDexesResourceWithRawResponse",
    "DexesResourceWithStreamingResponse",
    "AsyncDexesResourceWithStreamingResponse",
    "PoolsResource",
    "AsyncPoolsResource",
    "PoolsResourceWithRawResponse",
    "AsyncPoolsResourceWithRawResponse",
    "PoolsResourceWithStreamingResponse",
    "AsyncPoolsResourceWithStreamingResponse",
    "TokensResource",
    "AsyncTokensResource",
    "TokensResourceWithRawResponse",
    "AsyncTokensResourceWithRawResponse",
    "TokensResourceWithStreamingResponse",
    "AsyncTokensResourceWithStreamingResponse",
    "NetworksResource",
    "AsyncNetworksResource",
    "NetworksResourceWithRawResponse",
    "AsyncNetworksResourceWithRawResponse",
    "NetworksResourceWithStreamingResponse",
    "AsyncNetworksResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ticker_get_params import TickerGetParams as TickerGetParams
from .ticker_get_response import TickerGetResponse as TickerGetResponse
from .volume_chart_get_params import VolumeChartGetParams as VolumeChartGetParams
from .volume_chart_get_response import VolumeChartGetResponse as VolumeChartGetResponse
from .volume_chart_get_range_params import VolumeChartGetRangeParams as VolumeChartGetRangeParams
from .volume_chart_get_range_response import VolumeChartGetRangeResponse as VolumeChartGetRangeResponse
